@@ -66,7 +66,6 @@ import org.apache.cxf.jaxrs.utils.AnnotationUtils;
 import org.apache.cxf.jaxrs.utils.HttpUtils;
 import org.apache.cxf.jaxrs.utils.InjectionUtils;
 import org.apache.cxf.jaxrs.utils.JAXBUtils;
-import org.apache.cxf.jaxrs.utils.schemas.SchemaHandler;
 import org.apache.cxf.message.Attachment;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.staxutils.DepthExceededStaxException;
@@ -131,14 +130,6 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
         super.setProduceMediaTypes(types);
     }
     
-    public void setSchemas(List<String> locations) {
-        super.setSchemaLocations(locations);
-    }
-    
-    public void setSchemaHandler(SchemaHandler handler) {
-        super.setSchema(handler.getSchema());
-    }
-    
     public void setMarshallerProperties(Map<String, Object> marshallProperties) {
         mProperties = marshallProperties;
     }
@@ -150,7 +141,7 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
     public T readFrom(Class<T> type, Type genericType, Annotation[] anns, MediaType mt, 
         MultivaluedMap<String, String> headers, InputStream is) 
         throws IOException {
-        if (isPayloadEmpty()) {
+        if (isPayloadEmpty(headers)) {
             if (AnnotationUtils.getAnnotation(anns, Nullable.class) != null) {
                 return null;
             } else {
@@ -158,6 +149,7 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
             }
         }
         
+        XMLStreamReader reader = null;
         try {
             
             boolean isCollection = InjectionUtils.isSupportedCollectionOrArray(type);
@@ -170,10 +162,13 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
             if (JAXBElement.class.isAssignableFrom(type) 
                 || !isCollection && (unmarshalAsJaxbElement  
                 || jaxbElementClassMap != null && jaxbElementClassMap.containsKey(theType.getName()))) {
-                XMLStreamReader reader = getStreamReader(is, type, mt);
-                response = unmarshaller.unmarshal(
-                     TransformUtils.createNewReaderIfNeeded(reader, is), 
-                     theType);
+                reader = getStreamReader(is, type, mt);
+                reader = TransformUtils.createNewReaderIfNeeded(reader, is);
+                if (JAXBElement.class.isAssignableFrom(type) && type == theType) {
+                    response = unmarshaller.unmarshal(reader);
+                } else {
+                    response = unmarshaller.unmarshal(reader, theType);
+                }
             } else {
                 response = doUnmarshal(unmarshaller, type, is, mt);
             }
@@ -197,6 +192,8 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
         } catch (Exception e) {
             LOG.warning(getStackTrace(e));
             throw new WebApplicationException(e, Response.status(400).build());        
+        } finally {
+            StaxUtils.close(reader);
         }
         // unreachable
         return null;
@@ -206,7 +203,13 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
         throws JAXBException {
         XMLStreamReader reader = getStreamReader(is, type, mt);
         if (reader != null) {
-            return unmarshalFromReader(unmarshaller, reader, mt);
+            try {
+                return unmarshalFromReader(unmarshaller, reader, mt);
+            } catch (JAXBException e) {
+                throw e;
+            } finally {
+                StaxUtils.close(reader);
+            }
         }
         return unmarshalFromInputStream(unmarshaller, is, mt);
     }
@@ -243,15 +246,21 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
     protected Object unmarshalFromInputStream(Unmarshaller unmarshaller, InputStream is, MediaType mt) 
         throws JAXBException {
         // Try to create the read before unmarshalling the stream
-        if (is == null) {
-            Reader reader = getStreamHandlerFromCurrentMessage(Reader.class);
-            if (reader == null) {
-                LOG.severe("No InputStream, Reader, or XMStreamReader is available");
-                throw new WebApplicationException(500);
+        XMLStreamReader xmlReader = null;
+        try {
+            if (is == null) {
+                Reader reader = getStreamHandlerFromCurrentMessage(Reader.class);
+                if (reader == null) {
+                    LOG.severe("No InputStream, Reader, or XMStreamReader is available");
+                    throw new WebApplicationException(500);
+                }
+                xmlReader = StaxUtils.createXMLStreamReader(reader);
+            } else {
+                xmlReader = StaxUtils.createXMLStreamReader(is);
             }
-            return unmarshaller.unmarshal(StaxUtils.createXMLStreamReader(reader));
-        } else {
-            return unmarshaller.unmarshal(StaxUtils.createXMLStreamReader(is));
+            return unmarshaller.unmarshal(xmlReader);
+        } finally {
+            StaxUtils.close(xmlReader);
         }
     }
 
@@ -408,7 +417,11 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
             String value = pi.value();
             int ind = value.indexOf("href='");
             if (ind > 0) {
-                String relRef = value.substring(ind + 6, value.length() - 3);
+                String relRef = value.substring(ind + 6);
+                relRef = relRef.substring(0, relRef.length() - 3).trim();
+                if (relRef.endsWith("'")) {
+                    relRef = relRef.substring(0, relRef.length() - 1);
+                }
                 String absRef = buildAbsoluteXMLResourceURI(relRef);
                 value = value.substring(0, ind + 6) + absRef + "'?>";
             }
@@ -503,7 +516,9 @@ public class JAXBElementProvider<T> extends AbstractJAXBProvider<T>  {
                 mc.put(XMLStreamWriter.class.getName(), writer);    
             }
             marshalToWriter(ms, obj, writer, mt);
-            writer.writeEndDocument();
+            if (mc != null) { 
+                writer.writeEndDocument();
+            }
         } else {
             marshalToOutputStream(ms, obj, os, mt);
         }
