@@ -50,20 +50,26 @@ import org.apache.cxf.message.Message;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.Destination;
 import org.apache.cxf.transport.http.AbstractHTTPDestination;
+import org.apache.cxf.transport.servlet.BaseUrlHelper;
 
 public final class HttpUtils {
     
     private static final ResourceBundle BUNDLE = BundleUtils.getBundle(HttpUtils.class);
     private static final Logger LOG = LogUtils.getL7dLogger(HttpUtils.class);
     
-    private static final String ANY_IP_ADDRESS = "0.0.0.0";
-    private static final int DEFAULT_HTTP_PORT = 80;
+    private static final String REQUEST_PATH_TO_MATCH = "path_to_match";
+    private static final String REQUEST_PATH_TO_MATCH_SLASH = "path_to_match_slash";
     
+    private static final String HTTP_SCHEME = "http";
+    private static final String ANY_IP_ADDRESS = "0.0.0.0";
+    private static final String ANY_IP_ADDRESS_START = "://0.0.0.0";
+    private static final int DEFAULT_HTTP_PORT = 80;
+        
     private static final Pattern ENCODE_PATTERN = Pattern.compile("%[0-9a-fA-F][0-9a-fA-F]");
     private static final String CHARSET_PARAMETER = "charset";
     
     // there are more of such characters, ex, '*' but '*' is not affected by UrlEncode
-    private static final String PATH_RESERVED_CHARACTERS = "=@/:";
+    private static final String PATH_RESERVED_CHARACTERS = "=@/:!$&\'(),;~";
     private static final String QUERY_RESERVED_CHARACTERS = "?/";
     
     private HttpUtils() {
@@ -113,10 +119,15 @@ public final class HttpUtils {
     
     public static String urlEncode(String value) {
         
+        return urlEncode(value, "UTF-8");
+    }
+    
+    public static String urlEncode(String value, String enc) {
+        
         try {
-            value = URLEncoder.encode(value, "UTF-8");
+            value = URLEncoder.encode(value, enc);
         } catch (UnsupportedEncodingException ex) {
-            // unlikely to happen
+            throw new RuntimeException(ex);
         }
         
         return value;
@@ -182,13 +193,29 @@ public final class HttpUtils {
                || HttpHeaders.LAST_MODIFIED.equalsIgnoreCase(headerName); 
     }
     
+    public static boolean isHttpRequest(Message message) {
+        return message.get(AbstractHTTPDestination.HTTP_REQUEST) != null;
+    }
+    
+    public static URI toAbsoluteUri(String relativePath, Message message) {
+        String base = BaseUrlHelper.getBaseURL(
+            (HttpServletRequest)message.get(AbstractHTTPDestination.HTTP_REQUEST));
+        return URI.create(base + relativePath);
+    }
+    
     public static URI toAbsoluteUri(URI u, Message message) {
         HttpServletRequest request = 
             (HttpServletRequest)message.get(AbstractHTTPDestination.HTTP_REQUEST);
         boolean absolute = u.isAbsolute();
-        if (request != null && (!absolute || u.getHost().equals(ANY_IP_ADDRESS))) {
+        if (request != null && (!absolute || u.toString().contains(ANY_IP_ADDRESS_START))) {
             String serverAndPort = request.getServerName();
-            int port = request.getLocalPort();
+            boolean localAddressUsed = false;
+            if (absolute && ANY_IP_ADDRESS.equals(serverAndPort)) {
+                serverAndPort = request.getLocalAddr();
+                localAddressUsed = true;
+            }
+                
+            int port = localAddressUsed ? request.getLocalPort() : request.getServerPort();
             if (port != DEFAULT_HTTP_PORT) {
                 serverAndPort += ":" + port;
             }
@@ -196,22 +223,46 @@ public final class HttpUtils {
             if (!absolute) {
                 u = URI.create(base + u.toString());
             } else {
-                u = URI.create(u.toString().replace(ANY_IP_ADDRESS, serverAndPort));
+                int originalPort = u.getPort();
+                String replaceValue = originalPort == -1 ? ANY_IP_ADDRESS
+                    : ANY_IP_ADDRESS + ":" + originalPort;
+                u = URI.create(u.toString().replace(replaceValue, serverAndPort));
             }
         }
         return u;
     }
     
     public static String getPathToMatch(Message m, boolean addSlash) {
+        String var = addSlash ? REQUEST_PATH_TO_MATCH_SLASH : REQUEST_PATH_TO_MATCH;
+        String pathToMatch = (String)m.get(var);
+        if (pathToMatch != null) {
+            return pathToMatch; 
+        }
         String requestAddress = getProtocolHeader(m, Message.REQUEST_URI, "/");
+        if (m.get(Message.QUERY_STRING) == null) {
+            int index = requestAddress.lastIndexOf('?');
+            if (index > 0 && index < requestAddress.length()) {
+                m.put(Message.QUERY_STRING, requestAddress.substring(index + 1));
+                requestAddress = requestAddress.substring(0, index);
+            }
+        }
         String baseAddress = getBaseAddress(m);
-        return getPathToMatch(requestAddress, baseAddress, addSlash);
+        pathToMatch = getPathToMatch(requestAddress, baseAddress, addSlash);
+        m.put(var, pathToMatch);
+        return pathToMatch;
     }
     
     public static String getProtocolHeader(Message m, String name, String defaultValue) {
+        return getProtocolHeader(m, name, defaultValue, false);
+    }
+    
+    public static String getProtocolHeader(Message m, String name, String defaultValue, boolean setOnMessage) {
         String value = (String)m.get(name);
         if (value == null) {
             value = new HttpHeadersImpl(m).getRequestHeaders().getFirst(name);
+            if (value != null && setOnMessage) {
+                m.put(name, value);
+            }
         }
         return value == null ? defaultValue : value;
     }
@@ -219,7 +270,13 @@ public final class HttpUtils {
     public static String getBaseAddress(Message m) {
         String endpointAddress = getEndpointAddress(m);
         try {
-            String path = new URI(endpointAddress).getRawPath();
+            URI uri = new URI(endpointAddress);
+            String path = uri.getRawPath();
+            String scheme = uri.getScheme();
+            if (scheme != null && !scheme.startsWith(HttpUtils.HTTP_SCHEME)
+                && HttpUtils.isHttpRequest(m)) {
+                path = HttpUtils.toAbsoluteUri(path, m).getRawPath();
+            }
             return path.length() == 0 ? "/" : path;
         } catch (URISyntaxException ex) {
             return endpointAddress == null ? "/" : endpointAddress;
@@ -259,6 +316,8 @@ public final class HttpUtils {
             path = "/" + path;
         }
         m.put(Message.REQUEST_URI, baseAddress + path);
+        m.remove(REQUEST_PATH_TO_MATCH);
+        m.remove(REQUEST_PATH_TO_MATCH_SLASH);
     }
 
     
