@@ -24,8 +24,10 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.OutputStreamWriter;
 import java.io.Reader;
 import java.io.StringReader;
+import java.io.Writer;
 import java.net.URI;
 import java.net.URL;
 import java.util.ArrayList;
@@ -58,6 +60,7 @@ import javax.ws.rs.Produces;
 import javax.ws.rs.QueryParam;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.MultivaluedMap;
+import javax.ws.rs.core.PathSegment;
 import javax.ws.rs.core.Response;
 import javax.xml.namespace.QName;
 import javax.xml.transform.Source;
@@ -79,21 +82,21 @@ import org.apache.cxf.common.jaxb.JAXBUtils.SchemaCompiler;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.common.util.PackageUtils;
 import org.apache.cxf.common.util.ReflectionInvokationHandler;
+import org.apache.cxf.common.util.StringUtils;
 import org.apache.cxf.common.util.SystemPropertyAction;
 import org.apache.cxf.common.xmlschema.SchemaCollection;
 import org.apache.cxf.common.xmlschema.XmlSchemaConstants;
 import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.helpers.DOMUtils;
+import org.apache.cxf.helpers.JavaUtils;
 import org.apache.cxf.jaxrs.model.wadl.WadlGenerator;
 import org.apache.cxf.jaxrs.utils.JAXRSUtils;
 import org.apache.cxf.jaxrs.utils.ResourceUtils;
 import org.apache.cxf.service.model.SchemaInfo;
 import org.apache.cxf.staxutils.StaxUtils;
+import org.apache.cxf.tools.common.ToolException;
 import org.apache.ws.commons.schema.XmlSchema;
 
-/**
- * TODO: This will need to be moved into a separate module
- */
 public class SourceGenerator {
     public static final String CODE_TYPE_GRAMMAR = "grammar";
     public static final String CODE_TYPE_PROXY = "proxy";
@@ -110,8 +113,14 @@ public class SourceGenerator {
     private static final List<String> HTTP_OK_STATUSES =
         Arrays.asList(new String[] {"200", "201", "202", "203", "204"});
     
+    private static final Set<Class<?>> OPTIONAL_PARAMS = 
+        new HashSet<Class<?>>(Arrays.<Class<?>>asList(QueryParam.class, 
+                                                      HeaderParam.class,
+                                                      MatrixParam.class,
+                                                      FormParam.class));
     private static final Map<String, Class<?>> HTTP_METHOD_ANNOTATIONS;
     private static final Map<String, Class<?>> PARAM_ANNOTATIONS;
+    private static final String PLAIN_PARAM_STYLE = "plain";
     private static final Set<String> RESOURCE_LEVEL_PARAMS;
     private static final Map<String, String> AUTOBOXED_PRIMITIVES_MAP;
     private static final Map<String, String> XSD_SPECIFIC_TYPE_MAP;
@@ -146,8 +155,22 @@ public class SourceGenerator {
         
         XSD_SPECIFIC_TYPE_MAP = new HashMap<String, String>();
         XSD_SPECIFIC_TYPE_MAP.put("string", "String");
-        XSD_SPECIFIC_TYPE_MAP.put("decimal", "java.math.BigInteger");
         XSD_SPECIFIC_TYPE_MAP.put("integer", "long");
+        XSD_SPECIFIC_TYPE_MAP.put("int", "int");
+        XSD_SPECIFIC_TYPE_MAP.put("long", "long");
+        XSD_SPECIFIC_TYPE_MAP.put("byte", "byte");
+        XSD_SPECIFIC_TYPE_MAP.put("boolean", "boolean");
+        XSD_SPECIFIC_TYPE_MAP.put("unsignedInt", "long");
+        XSD_SPECIFIC_TYPE_MAP.put("unsignedShort", "int");
+        XSD_SPECIFIC_TYPE_MAP.put("unsignedByte", "short");
+        XSD_SPECIFIC_TYPE_MAP.put("unsignedLong", "java.math.BigInteger");
+        XSD_SPECIFIC_TYPE_MAP.put("decimal", "java.math.BigInteger");
+        XSD_SPECIFIC_TYPE_MAP.put("positiveInteger", "java.math.BigInteger");
+        XSD_SPECIFIC_TYPE_MAP.put("QName", "javax.xml.namespace.QName");
+        XSD_SPECIFIC_TYPE_MAP.put("duration", "javax.xml.datatype.Duration");
+        XSD_SPECIFIC_TYPE_MAP.put("date", "java.util.Date");
+        XSD_SPECIFIC_TYPE_MAP.put("dateTime", "java.util.Date");
+        XSD_SPECIFIC_TYPE_MAP.put("time", "java.util.Date");
     }
 
     private Comparator<String> importsComparator;
@@ -158,6 +181,9 @@ public class SourceGenerator {
     private String wadlPath;
     private String wadlNamespace = WadlGenerator.WADL_NS;
     private boolean generateEnums;
+    private boolean skipSchemaGeneration;
+    private boolean inheritResourceParams;
+    private boolean useVoidForEmptyResponses = true;
     
     private Map<String, String> properties; 
     
@@ -167,10 +193,12 @@ public class SourceGenerator {
     private List<InputSource> schemaPackageFiles = Collections.emptyList();
     private List<String> compilerArgs = new ArrayList<String>();
     private Map<String, String> schemaPackageMap = Collections.emptyMap();
-    private Map<String, String> schemaTypesMap = Collections.emptyMap();
+    private Map<String, String> javaTypeMap = Collections.emptyMap();
+    private Map<String, String> schemaTypeMap = Collections.emptyMap();
+    private Map<String, String> mediaTypesMap = Collections.emptyMap();
     private Bus bus;
     private boolean supportMultipleXmlReps;
-    
+        
     private SchemaCollection schemaCollection = new SchemaCollection();
     
     public SourceGenerator() {
@@ -189,6 +217,9 @@ public class SourceGenerator {
         this.wadlNamespace = ns;
     }
     
+    public void setUseVoidForEmptyResponses(boolean use) {
+        this.useVoidForEmptyResponses = use;
+    }
     public String getWadlNamespace() {
         return wadlNamespace;
     }
@@ -197,6 +228,10 @@ public class SourceGenerator {
         this.generateEnums = generate;
     }
     
+    public void setSkipSchemaGeneration(boolean skip) {
+        this.skipSchemaGeneration = skip;
+    }
+    
     private String getClassPackageName(String wadlPackageName) {
         if (resourcePackageName != null) {
             return resourcePackageName;
@@ -229,8 +264,9 @@ public class SourceGenerator {
     
     private GrammarInfo generateSchemaCodeAndInfo(Application app, Set<String> typeClassNames, 
                                                   File srcDir) {
+        
         List<SchemaInfo> schemaElements = getSchemaElements(app);
-        if (schemaElements != null && !schemaElements.isEmpty()) {
+        if (!skipSchemaGeneration && schemaElements != null && !schemaElements.isEmpty()) {
             // generate classes from schema
             JCodeModel codeModel = createCodeModel(schemaElements, typeClassNames);
             if (codeModel != null) {
@@ -355,8 +391,7 @@ public class SourceGenerator {
         for (Element el : elementEls) {
             String type = el.getAttribute("type");
             if (type.length() > 0) {
-                String[] pair = type.split(":");
-                elementTypeMap.put(el.getAttribute("name"), pair.length == 1 ? pair[0] : pair[1]);
+                elementTypeMap.put(el.getAttribute("name"), type);
             }
         }
         Element includeEl = DOMUtils.getFirstChildWithName(schemaEl, 
@@ -531,15 +566,23 @@ public class SourceGenerator {
                               String currentPath) {
     //CHECKSTYLE:ON    
         List<Element> methodEls = getWadlElements(rElement, "method");
-       
+        
+        List<Element> currentInheritedParams = inheritResourceParams 
+            ? new LinkedList<Element>(info.getInheritedParams()) : Collections.<Element>emptyList();
         for (Element methodEl : methodEls) {
             writeResourceMethod(methodEl, classPackage, imports, sbCode, info, isRoot, currentPath);    
         }
+        if (inheritResourceParams && methodEls.isEmpty()) {
+            info.getInheritedParams().addAll(getWadlElements(rElement, "param"));
+        }
         
         List<Element> childEls = getWadlElements(rElement, "resource");
         for (Element childEl : childEls) {
             String path = childEl.getAttribute("path");
-            String newPath = (currentPath + path).replace("//", "/");
+            if (!path.startsWith("/")) {
+                path = "/" + path;
+            }
+            String newPath = currentPath + path.replace("//", "/");
             String id = childEl.getAttribute("id");
             if (id.length() == 0) {
                 writeMethods(childEl, classPackage, imports, sbCode, info, id, false, newPath);
@@ -547,6 +590,8 @@ public class SourceGenerator {
                 writeResourceMethod(childEl, classPackage, imports, sbCode, info, false, newPath);
             }
         }
+        info.getInheritedParams().clear();
+        info.getInheritedParams().addAll(currentInheritedParams);
     }
     
     private void writeAnnotation(StringBuilder sbCode, Set<String> imports,
@@ -613,6 +658,7 @@ public class SourceGenerator {
             if (id.length() == 0) {
                 id = methodNameLowerCase;
             }
+            
             String suffixName = "";
             if (!jaxpSourceRequired && inXmlRep != null && xmlRequestReps.size() > 1) {
                 String value = inXmlRep.getAttribute("element");
@@ -648,7 +694,17 @@ public class SourceGenerator {
                 responseTypeAvailable = writeResponseType(responseEls, sbCode, imports, info);
                 String genMethodName = id + suffixName;
                 if (methodNameLowerCase.equals(genMethodName)) {
-                    genMethodName += firstCharToUpperCase(currentPath.replaceAll("/", ""));
+                    List<PathSegment> segments = JAXRSUtils.getPathSegments(currentPath, true, true);
+                    StringBuilder sb = new StringBuilder();
+                    for (PathSegment ps : segments) {
+                        String pathSeg = ps.getPath().replaceAll("\\{", "").replaceAll("\\}", "");
+                        int index = pathSeg.indexOf(":");
+                        if (index > 0) {
+                            pathSeg = pathSeg.substring(0, index);
+                        }
+                        sb.append(pathSeg);
+                    }
+                    genMethodName += firstCharToUpperCase(sb.toString());
                 }
                 sbCode.append(genMethodName);
             } else {
@@ -675,7 +731,7 @@ public class SourceGenerator {
             
             sbCode.append("(");
             
-            List<Element> inParamElements = getParameters(resourceEl, 
+            List<Element> inParamElements = getParameters(resourceEl, info.getInheritedParams(),
                         !isRoot && !isResourceElement && resourceEl.getAttribute("id").length() > 0);
             
             Element repElement = getActualRepElement(allRequestReps, inXmlRep); 
@@ -707,15 +763,24 @@ public class SourceGenerator {
         return xmlReps;
     }
     
-    private List<Element> getParameters(Element resourceEl, boolean isSubresourceMethod) {
+    private List<Element> getParameters(Element resourceEl, List<Element> inheritedParams, 
+                                        boolean isSubresourceMethod) {
         List<Element> inParamElements = new LinkedList<Element>();
         List<Element> allParamElements = getWadlElements(resourceEl, "param");
+        List<Element> newInheritedParams = inheritResourceParams ? new LinkedList<Element>() 
+            : Collections.<Element>emptyList();
         for (Element el : allParamElements) {
-            if (isSubresourceMethod && RESOURCE_LEVEL_PARAMS.contains(el.getAttribute("style"))) {
+            boolean isResourceLevelParam = RESOURCE_LEVEL_PARAMS.contains(el.getAttribute("style")); 
+            if (isSubresourceMethod && isResourceLevelParam) {
                 continue;
             }
+            if (inheritResourceParams && isResourceLevelParam) {
+                newInheritedParams.add(el);
+            }
             inParamElements.add(el);
         }
+        inParamElements.addAll(inheritedParams);
+        inheritedParams.addAll(newInheritedParams);
         return inParamElements;
     }
 
@@ -762,7 +827,7 @@ public class SourceGenerator {
                 return el;
             }
         }
-        return null;
+        return repElements.isEmpty() ? null : repElements.get(0);
     }
     
     private boolean writeResponseType(List<Element> responseEls,
@@ -779,8 +844,13 @@ public class SourceGenerator {
             repElements = CastUtils.cast(Collections.emptyList(), Element.class);
         }
         
-        if (repElements.size() == 0) {    
-            sbCode.append("void ");
+        if (repElements.size() == 0) {
+            if (useVoidForEmptyResponses) {
+                sbCode.append("void ");
+            } else {
+                addImport(imports, Response.class.getName());
+                sbCode.append("Response ");
+            }
             return false;
         }
         String elementName = getElementRefName(
@@ -839,7 +909,8 @@ public class SourceGenerator {
         for (int i = 0; i < inParamEls.size(); i++) {
     
             Element paramEl = inParamEls.get(i);
-            Class<?> paramAnn = PARAM_ANNOTATIONS.get(paramEl.getAttribute("style"));
+            
+            Class<?> paramAnn = getParamAnnotation(paramEl.getAttribute("style"));
             if (paramAnn == QueryParam.class && form) {
                 paramAnn = FormParam.class; 
             } 
@@ -865,7 +936,7 @@ public class SourceGenerator {
             boolean isRepeating = Boolean.valueOf(paramEl.getAttribute("repeating"));
             String type = enumCreated ? getTypicalClassName(name)
                 : getPrimitiveType(paramEl, info, imports);
-            if (paramAnn == QueryParam.class
+            if (OPTIONAL_PARAMS.contains(paramAnn)
                 && (isRepeating || !Boolean.valueOf(paramEl.getAttribute("required")))    
                 && AUTOBOXED_PRIMITIVES_MAP.containsKey(type)) {
                 type = AUTOBOXED_PRIMITIVES_MAP.get(type);
@@ -874,7 +945,12 @@ public class SourceGenerator {
                 addImport(imports, List.class.getName());
                 type = "List<" + type + ">";
             }
-            String paramName = name.replaceAll("[\\.\\-]", "_");
+            String paramName;
+            if (JavaUtils.isJavaKeyword(name)) {
+                paramName = name.concat("_arg");
+            } else {
+                paramName = name.replaceAll("[:\\.\\-]", "_");
+            }
             sbCode.append(type).append(" ").append(paramName);
             if (i + 1 < inParamEls.size()) {
                 sbCode.append(", ");
@@ -889,6 +965,10 @@ public class SourceGenerator {
             if (!jaxpRequired) {    
                 elementParamType = getElementRefName(repElement, info, imports);
                 if (elementParamType != null) {
+                    int lastIndex = elementParamType.lastIndexOf('.');
+                    if (lastIndex != -1) {
+                        elementParamType = elementParamType.substring(lastIndex + 1);
+                    }
                     elementParamName = elementParamType.toLowerCase();
                 }
             } else {
@@ -909,6 +989,18 @@ public class SourceGenerator {
         }
     }
     
+    private Class<?> getParamAnnotation(String paramStyle) {
+        Class<?> paramAnn = PARAM_ANNOTATIONS.get(paramStyle);
+        if (paramAnn == null) {
+            String error = "Unsupported parameter style: " + paramStyle;
+            if (PLAIN_PARAM_STYLE.equals(paramStyle)) {
+                error += ", plain style parameters have to be wrapped by representations";    
+            }
+            throw new ToolException(error); 
+        }
+        return paramAnn;
+    }
+    
     private void generateEnumClass(String clsName, List<Element> options, File src, String classPackage) {
         StringBuilder sbImports = new StringBuilder();
         StringBuilder sbCode = new StringBuilder();
@@ -1006,21 +1098,29 @@ public class SourceGenerator {
     }
     
     private String getPrimitiveType(Element paramEl, ContextInfo info, Set<String> imports) {
+        final String defaultValue = "String";
         String type = paramEl.getAttribute("type");
         if (type.length() == 0) {
-            return "String";
+            return defaultValue;
         }
+        
         String[] pair = type.split(":");
-        String value = pair.length == 2 ? pair[1] : type;
-        if (XSD_SPECIFIC_TYPE_MAP.containsKey(value)) {
-            return XSD_SPECIFIC_TYPE_MAP.get(value);
-        } else {
-            String actualValue = value.replaceAll("[\\-\\_]", "");
-            if (pair.length > 1) {
-                actualValue = convertRefToClassName(pair[0], actualValue, "String", info, imports);
+        if (pair.length == 2) {
+            if (XSD_SPECIFIC_TYPE_MAP.containsKey(pair[1])) {
+                String expandedName = "{" + XmlSchemaConstants.XSD_NAMESPACE_URI + "}" + pair[1];
+                if (schemaTypeMap.containsKey(expandedName)) {
+                    return schemaTypeMap.get(expandedName);
+                }
+                
+                return XSD_SPECIFIC_TYPE_MAP.get(pair[1]);
             }
-            return actualValue;
+            
+            String value = pair[1].replaceAll("[\\-\\_]", "");
+            return convertRefToClassName(pair[0], value, defaultValue, info, imports);
+        } else {
+            return type;
         }
+        
     }
     
     private String convertRefToClassName(String prefix,
@@ -1032,23 +1132,33 @@ public class SourceGenerator {
         if (gInfo != null) {
             String namespace = gInfo.getNsMap().get(prefix);
             if (namespace != null) {
+                                
                 String packageName = getPackageFromNamespace(namespace);
                 String clsName = getSchemaClassName(packageName, gInfo, actualValue, 
                                                     info.getTypeClassNames());
-                if (clsName != null) {
-                    addImport(imports, clsName);
-                    int index = clsName.lastIndexOf(".");
-                    
-                    if (index != -1) {
-                        clsName = clsName.substring(index + 1);
-                    } 
-                    return clsName;      
+                
+                if (clsName == null) {
+                    clsName = schemaTypeMap.get("{" + namespace + "}" + actualValue);
                 }
+                if (clsName != null) {
+                    return addImportsAndGetSimpleName(imports, clsName);
+                }
+                
             }
         }
         return defaultValue;
     }
     
+    private String addImportsAndGetSimpleName(Set<String> imports, String clsName) {
+        addImport(imports, clsName);
+        int index = clsName.lastIndexOf(".");
+        
+        if (index != -1) {
+            clsName = clsName.substring(index + 1);
+        }
+        return clsName;
+    }
+    
     private String getElementRefName(Element repElement,
                                      ContextInfo info, 
                                      Set<String> imports) {
@@ -1063,6 +1173,12 @@ public class SourceGenerator {
                 return convertRefToClassName(pair[0], pair[1], null, info, imports);
             }
         } else {
+            // try mediaTypesMap first
+            String mediaType = repElement.getAttribute("mediaType");
+            if (!StringUtils.isEmpty(mediaType) && mediaTypesMap.containsKey(mediaType)) {
+                return addImportsAndGetSimpleName(imports, mediaTypesMap.get(mediaType));
+            }
+            
             Element param = DOMUtils.getFirstChildWithName(repElement, getWadlNamespace(), "param");
             if (param != null) {
                 return getPrimitiveType(param, info, imports);
@@ -1075,14 +1191,26 @@ public class SourceGenerator {
                                       Set <String> typeClassNames) {
         String clsName = matchClassName(typeClassNames, packageName, localName);
         if (clsName == null && gInfo != null) {
-            String elementTypeName = gInfo.getElementTypeMap().get(localName);
-            clsName = matchClassName(typeClassNames, packageName, elementTypeName);
-            if (clsName == null && elementTypeName != null && elementTypeName.contains("_")) {
-                clsName = matchClassName(typeClassNames, packageName, elementTypeName.replaceAll("_", ""));
+            String prefixedElementTypeName = gInfo.getElementTypeMap().get(localName);
+            if (prefixedElementTypeName != null) {
+                String[] pair = prefixedElementTypeName.split(":");
+                String elementTypeName = pair.length == 2 ? pair[1] : pair[0];
+                clsName = matchClassName(typeClassNames, packageName, elementTypeName);
+                if (clsName == null && elementTypeName.contains("_")) {
+                    clsName = matchClassName(typeClassNames, packageName, elementTypeName.replaceAll("_", ""));
+                }
+                if (clsName == null && pair.length == 2) {
+                    String namespace = gInfo.getNsMap().get(pair[0]);
+                    if (namespace != null) {
+                        packageName = getPackageFromNamespace(namespace);
+                        clsName = matchClassName(typeClassNames, packageName, elementTypeName);
+                    }
+                }
+                
             }
         }
-        if (clsName == null && schemaTypesMap != null) {
-            clsName = schemaTypesMap.get(packageName + "." + localName);
+        if (clsName == null && javaTypeMap != null) {
+            clsName = javaTypeMap.get(packageName + "." + localName);
         }
         return clsName;
     }
@@ -1150,9 +1278,16 @@ public class SourceGenerator {
         
         try {
             file.createNewFile();
-            FileOutputStream fos = new FileOutputStream(file);
-            fos.write(content.getBytes());
-            fos.close();
+            Writer writer = null;
+            try {
+                writer = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");
+                writer.write(content);
+                writer.flush();
+            } finally {
+                if (writer != null) {
+                    writer.close();
+                }
+            }
         } catch (FileNotFoundException ex) {
             LOG.warning(file.getAbsolutePath() + " is not found");
         } catch (IOException ex) {
@@ -1207,10 +1342,14 @@ public class SourceGenerator {
             if (schemaURI == null) {
                 if (!URI.create(href).isAbsolute() && app.getWadlPath() != null) {
                     String baseWadlPath = getBaseWadlPath(app.getWadlPath());
-                    if  (!href.startsWith("/")) {
+                    if  (!href.startsWith("/") && !href.contains("..")) {
                         schemaURI = baseWadlPath + href;
                     } else {
-                        schemaURI = URI.create(baseWadlPath).resolve(href).toString();
+                        try {
+                            schemaURI = new URL(new URL(baseWadlPath), href).toString();
+                        } catch (Exception ex) {
+                            schemaURI = URI.create(baseWadlPath).resolve(href).toString();
+                        }
                     }
                 } else {
                     schemaURI = href;
@@ -1279,6 +1418,10 @@ public class SourceGenerator {
     private JCodeModel createCodeModel(List<SchemaInfo> schemaElements, Set<String> type) {
         
         SchemaCompiler compiler = createCompiler(type);
+        Object elForRun = ReflectionInvokationHandler
+            .createProxyWrapper(new InnerErrorListener(),
+                            JAXBUtils.getParamClass(compiler, "setErrorListener"));
+        compiler.setErrorListener(elForRun);
         compiler.setEntityResolver(OASISCatalogManager.getCatalogManager(bus)
                                        .getEntityResolver());
         if (compilerArgs.size() > 0) {
@@ -1290,11 +1433,6 @@ public class SourceGenerator {
             compiler.getOptions().addBindFile(is);
         }
         
-        Object elForRun = ReflectionInvokationHandler
-            .createProxyWrapper(new InnerErrorListener(),
-                            JAXBUtils.getParamClass(compiler, "setErrorListener"));
-        
-        compiler.setErrorListener(elForRun);
         S2JJAXBModel intermediateModel = compiler.bind();
         JCodeModel codeModel = intermediateModel.generateCode(null, elForRun);
         JAXBUtils.logGeneratedClassNames(LOG, codeModel);
@@ -1383,12 +1521,24 @@ public class SourceGenerator {
         this.compilerArgs = args;
     }
 
+    public void setInheritResourceParams(boolean inherit) {
+        this.inheritResourceParams = inherit;
+    }
+    
     public void setSchemaPackageMap(Map<String, String> map) {
         this.schemaPackageMap = map;
     }
     
-    public void setSchemaTypesMap(Map<String, String> map) {
-        this.schemaTypesMap = map;
+    public void setJavaTypeMap(Map<String, String> map) {
+        this.javaTypeMap = map;
+    }
+    
+    public void setSchemaTypeMap(Map<String, String> map) {
+        this.schemaTypeMap = map;
+    }
+    
+    public void setMediaTypeMap(Map<String, String> map) {
+        this.mediaTypesMap = map;
     }
     
     public void setBus(Bus bus) {
@@ -1486,6 +1636,7 @@ public class SourceGenerator {
         private Set<String> resourceClassNames = new HashSet<String>();
         private Application rootApp;
         private File srcDir;
+        private List<Element> inheritedParams = new LinkedList<Element>();
         
         public ContextInfo(Application rootApp,
                            File srcDir,
@@ -1498,6 +1649,9 @@ public class SourceGenerator {
             this.rootApp = rootApp;
             this.srcDir = srcDir;
         }
+        public List<Element> getInheritedParams() {
+            return inheritedParams;
+        }
         public Application getApp() {
             return rootApp;
         }
