@@ -50,6 +50,7 @@ import org.apache.cxf.ws.security.policy.model.SecureConversationToken;
 import org.apache.cxf.ws.security.policy.model.SecurityContextToken;
 import org.apache.cxf.ws.security.policy.model.SignedEncryptedElements;
 import org.apache.cxf.ws.security.policy.model.SignedEncryptedParts;
+import org.apache.cxf.ws.security.policy.model.SpnegoContextToken;
 import org.apache.cxf.ws.security.policy.model.SupportingToken;
 import org.apache.cxf.ws.security.policy.model.Token;
 import org.apache.cxf.ws.security.policy.model.TokenWrapper;
@@ -62,6 +63,7 @@ import org.apache.ws.security.WSConstants;
 import org.apache.ws.security.WSEncryptionPart;
 import org.apache.ws.security.WSPasswordCallback;
 import org.apache.ws.security.WSSConfig;
+import org.apache.ws.security.WSSecurityException;
 import org.apache.ws.security.components.crypto.Crypto;
 import org.apache.ws.security.conversation.ConversationConstants;
 import org.apache.ws.security.message.WSSecDKSign;
@@ -117,9 +119,9 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
         
     }
     
-    private static void addSig(List<byte[]> signatureValues, byte[] val) {
-        if (val != null) {
-            signatureValues.add(val);
+    private void addSig(byte[] val) {
+        if (val != null && val.length > 0) {
+            signatures.add(val);
         }
     }
     
@@ -215,7 +217,6 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
      */
     private void handleEndorsingSupportingTokens() throws Exception {
         Collection<AssertionInfo> ais;
-        List<byte[]> signatureValues = new ArrayList<byte[]>();
         
         ais = aim.get(SP12Constants.SIGNED_ENDORSING_SUPPORTING_TOKENS);
         if (ais != null) {
@@ -226,7 +227,7 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
             }
             if (sgndSuppTokens != null) {
                 for (Token token : sgndSuppTokens.getTokens()) {
-                    handleEndorsingToken(token, sgndSuppTokens, signatureValues);
+                    handleEndorsingToken(token, sgndSuppTokens);
                 }
             }
         }
@@ -241,7 +242,7 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
             
             if (endSuppTokens != null) {
                 for (Token token : endSuppTokens.getTokens()) {
-                    handleEndorsingToken(token, endSuppTokens, signatureValues);
+                    handleEndorsingToken(token, endSuppTokens);
                 }
             }
         }
@@ -255,7 +256,7 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
             
             if (endSuppTokens != null) {
                 for (Token token : endSuppTokens.getTokens()) {
-                    handleEndorsingToken(token, endSuppTokens, signatureValues);
+                    handleEndorsingToken(token, endSuppTokens);
                 }
             }
         }
@@ -269,38 +270,29 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
             
             if (endSuppTokens != null) {
                 for (Token token : endSuppTokens.getTokens()) {
-                    handleEndorsingToken(token, endSuppTokens, signatureValues);
+                    handleEndorsingToken(token, endSuppTokens);
                 }
             }
         }
     }
     
     private void handleEndorsingToken(
-        Token token, SupportingToken wrapper, List<byte[]> signatureValues
+        Token token, SupportingToken wrapper
     ) throws Exception {
         if (token instanceof IssuedToken
             || token instanceof SecureConversationToken
             || token instanceof SecurityContextToken
-            || token instanceof KeyValueToken
-            || token instanceof KerberosToken) {
-            addSig(
-                signatureValues, 
-                doIssuedTokenSignature(token, wrapper)
-            );
+            || token instanceof KerberosToken
+            || token instanceof SpnegoContextToken) {
+            addSig(doIssuedTokenSignature(token, wrapper));
         } else if (token instanceof X509Token
             || token instanceof KeyValueToken) {
-            addSig(
-                signatureValues, 
-                doX509TokenSignature(token, wrapper)
-            );
+            addSig(doX509TokenSignature(token, wrapper));
         } else if (token instanceof SamlToken) {
             AssertionWrapper assertionWrapper = addSamlToken((SamlToken)token);
             assertionWrapper.toDOM(saaj.getSOAPPart());
             storeAssertionAsSecurityToken(assertionWrapper);
-            addSig(
-                signatureValues, 
-                doIssuedTokenSignature(token, wrapper)
-            );
+            addSig(doIssuedTokenSignature(token, wrapper));
         } else if (token instanceof UsernameToken) {
             // Create a UsernameToken object for derived keys and store the security token
             WSSecUsernameToken usernameToken = addDKUsernameToken((UsernameToken)token, true);
@@ -316,10 +308,7 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
             getTokenStore().add(tempTok);
             message.setContextualProperty(SecurityConstants.TOKEN_ID, tempTok.getId());
             
-            addSig(
-                signatureValues, 
-                doIssuedTokenSignature(token, wrapper)
-            );
+            addSig(doIssuedTokenSignature(token, wrapper));
         }
     }
     
@@ -387,6 +376,10 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
         boolean tokenIncluded = false;
         // Get the issued token
         SecurityToken secTok = getSecurityToken();
+        if (secTok == null) {
+            LOG.fine("The retrieved SecurityToken was null");
+            throw new WSSecurityException("The retrieved SecurityToken was null");
+        }
         
         if (includeToken(token.getInclusion())) {
             //Add the token
@@ -600,7 +593,7 @@ public class TransportBindingHandler extends AbstractBindingBuilder {
                 result.addAll(
                     this.getElements(
                         "Element", signedElements.getXPathExpressions(), 
-                        signedElements.getDeclaredNamespaces(), found
+                        signedElements.getDeclaredNamespaces(), found, true
                     )
                 );
             } catch (XPathExpressionException e) {
